@@ -319,7 +319,8 @@ qmake -config "${BUILD_MODE}" -spec macx-g++ "${PROJECT}" \
       QMAKE_MACOSX_DEPLOYMENT_TARGET="${MACOSX_DEPLOYMENT_TARGET}" \
       OSX_STDLIB="${STDLIB}" \
       MACPORTS_INCLUDE_PATH="${MACPORTS_PREFIX}/include" \
-      MACPORTS_LIBRARY_PATH="${MACPORTS_PREFIX}/lib"
+      MACPORTS_LIBRARY_PATH="${MACPORTS_PREFIX}/lib" \
+      MACPORTS_PREFIX="${MACPORTS_PREFIX}"
 
 phase "Running make"
 make -j2
@@ -589,8 +590,8 @@ if [ "${BUNDLE}" = "1" ]; then
     --copy "${TOP_DIR}/res/osxbundle/macdmg.DS_Store:/.DS_Store" \
     --copy "${TOP_DIR}/LICENSE" \
     --copy "${TOP_DIR}/COPYING" \
-    --symlink "/Applications: " \
-    --icon "${TOP_DIR}/res/img/icons/x2go-mac.icns" \
+    --symlink "/Applications" \
+    --icon "${TOP_DIR}/res/img/icons/dmg/x2go-mac-dmg.icns" \
     --format "UDBZ"
 fi
 
