@@ -36,6 +36,7 @@
 #include "onmainwindow.h"
 #include <QTemporaryFile>
 #include <QInputDialog>
+#include "InteractionDialog.h"
 
 
 HttpBrokerClient::HttpBrokerClient ( ONMainWindow* wnd, ConfigFile* cfg )
@@ -99,6 +100,18 @@ void HttpBrokerClient::createSshConnection()
               SLOT ( slotSshConnectionError ( QString,QString ) ) );
     connect ( sshConnection, SIGNAL(ioErr(SshProcess*,QString,QString)), this,
               SLOT(slotSshIoErr(SshProcess*,QString,QString)));
+
+
+    connect ( sshConnection, SIGNAL(startInteraction(SshMasterConnection*,QString)),mainWindow,
+              SLOT(slotSshInteractionStart(SshMasterConnection*,QString)) );
+    connect ( sshConnection, SIGNAL(updateInteraction(SshMasterConnection*,QString)),mainWindow,
+              SLOT(slotSshInteractionUpdate(SshMasterConnection*,QString)) );
+    connect ( sshConnection, SIGNAL(finishInteraction(SshMasterConnection*)),mainWindow,
+              SLOT(slotSshInteractionFinish(SshMasterConnection*)));
+    connect ( mainWindow->getInteractionDialog(), SIGNAL(textEntered(QString)), sshConnection,
+              SLOT(interactionTextEnter(QString)));
+    connect ( mainWindow->getInteractionDialog(), SIGNAL(interrupt()), sshConnection, SLOT(interactionInterruptSlot()));
+
     sshConnection->start();
 }
 
@@ -118,6 +131,7 @@ void HttpBrokerClient::slotSshConnectionError(QString message, QString lastSessi
 
 void HttpBrokerClient::slotSshConnectionOk()
 {
+    mainWindow->getInteractionDialog()->hide();
     getUserSessions();
 }
 
@@ -141,7 +155,7 @@ void HttpBrokerClient::slotSshServerAuthError(int error, QString sshMessage, Ssh
         return;
 
     case SSH_SERVER_FOUND_OTHER:
-        errMsg=tr ( "The host key for this server was not found but another"
+        errMsg=tr ( "The host key for this server was not found but another "
                     "type of key exists. An attacker might have changed the default server key to "
                     "trick your client into thinking the key does not exist yet." );
         connection->writeKnownHosts(false);
@@ -167,7 +181,7 @@ void HttpBrokerClient::slotSshServerAuthError(int error, QString sshMessage, Ssh
         slotSshUserAuthError ( sshMessage );
         return ;
     case SSH_SERVER_FILE_NOT_FOUND:
-        errMsg=tr ( "Could not find known hosts file."
+        errMsg=tr ( "Could not find known hosts file. "
                     "If you accept the host key here, the file will be automatically created." );
         break;
 
@@ -225,6 +239,12 @@ void HttpBrokerClient::slotSshServerAuthPassphrase(SshMasterConnection* connecti
 
 }
 
+void HttpBrokerClient::closeSSHInteractionDialog()
+{
+    slotSshUserAuthError("NO_ERROR");
+}
+
+
 void HttpBrokerClient::slotSshUserAuthError(QString error)
 {
     if ( sshConnection )
@@ -233,10 +253,13 @@ void HttpBrokerClient::slotSshUserAuthError(QString error)
         delete sshConnection;
         sshConnection=0l;
     }
+    mainWindow->getInteractionDialog()->hide();
 
-    QMessageBox::critical ( 0l,tr ( "Authentication failed." ),error,
-                            QMessageBox::Ok,
-                            QMessageBox::NoButton );
+    if(error!="NO_ERROR")
+
+        QMessageBox::critical ( 0l,tr ( "Authentication failed." ),error,
+                                QMessageBox::Ok,
+                                QMessageBox::NoButton );
     emit authFailed();
     return;
 }
