@@ -44,6 +44,10 @@
 #include <QTranslator>
 #include <QLocale>
 #include <QProcessEnvironment>
+#include <QDirIterator>
+#include <QApplication>
+#include <QDesktopWidget>
+#include <libssh/callbacks.h>
 #include "sshmasterconnection.h"
 #include "non_modal_messagebox.h"
 
@@ -88,6 +92,7 @@ class QStandardItemModel;
 class HttpBrokerClient;
 class QMenu;
 class QComboBox;
+class InteractionDialog;
 
 class SessionExplorer;
 struct user
@@ -341,6 +346,10 @@ public:
     void suspendSession ( QString sessId );
     bool termSession ( QString sessId,
                        bool warn=true );
+    InteractionDialog* getInteractionDialog()
+    {
+      return interDlg;
+    }
     void setStatStatus ( QString status=QString::null );
     x2goSession getNewSessionFromString ( const QString& string );
     void runCommand();
@@ -535,6 +544,17 @@ public:
         return &config;
     }
 
+#if defined (Q_OS_DARWIN) || defined (Q_OS_WIN)
+    bool getSystemDisablePARecord()
+    {
+        return systemDisablePARecord;
+    }
+    bool getSystemDisablePA()
+    {
+        return systemDisablePA;
+    }
+#endif /* defined (Q_OS_DARWIN) || defined (Q_OS_WIN) */
+
     void runApplication(QString exec);
 
 
@@ -577,6 +597,7 @@ public:
 
 
 private:
+    InteractionDialog* interDlg;
     QString m_x2goconfig;
     QStringList _internApplicationsNames;
     QStringList _transApplicationsNames;
@@ -592,6 +613,7 @@ private:
     bool keepTrayIcon;
     bool hideFolderSharing;
     bool brokerNoauthWithSessionUsername;
+    bool brokerCredsForSession;
     bool defaultUseSound;
     bool defaultXinerama;
     bool cardStarted;
@@ -847,6 +869,8 @@ private:
 #if defined (Q_OS_DARWIN) || defined (Q_OS_WIN)
     QThread *pulseManagerThread;
     PulseManager *pulseManager;
+    bool systemDisablePARecord;
+    bool systemDisablePA;
 #endif /* defined (Q_OS_DARWIN) || defined (Q_OS_WIN) */
 
 
@@ -1022,7 +1046,11 @@ private slots:
     void slotSshConnectionError ( QString message, QString lastSessionError );
     void slotSshServerAuthError ( int error, QString sshMessage, SshMasterConnection* connection );
     void slotSshServerAuthPassphrase ( SshMasterConnection* connection, bool verificationCode );
+    void slotSshInteractionStart ( SshMasterConnection* connection, QString prompt );
+    void slotSshInteractionUpdate ( SshMasterConnection* connection, QString output );
+    void slotSshInteractionFinish ( SshMasterConnection* connection);
     void slotSshServerAuthChallengeResponse( SshMasterConnection* connection, QString Challenge );
+    void slotCloseInteractionDialog();
     void slotSshUserAuthError ( QString error );
     void slotSshConnectionOk();
     void slotServSshConnectionOk(QString server);
@@ -1040,6 +1068,7 @@ public slots:
     void setFocus();
 
 private slots:
+    void slotShowPAMSGDialog(bool error, const QString& main_text, const QString& info_text, bool modal);
     void slotSnameChanged ( const QString& );
     void slotSelectedFromList ( SessionButton* session );
     void slotSessEnter();
@@ -1080,6 +1109,7 @@ private slots:
     void slotShowAdvancedStat();
     void slotRestartProxy();
     void slotTestSessionStatus();
+    void SlotRunCommand(bool, QString output, int);
     void slotRetRunCommand ( bool result, QString output,
                              int );
     void slotGetServers ( bool result, QString output,
@@ -1140,6 +1170,7 @@ private slots:
     void slotReconnectSession();
     void slotStartBroker();
     void slotStartNewBrokerSession ();
+    void slotInitLibssh ();
 #ifdef Q_OS_DARWIN
     void slotSetModMap();
     void handle_xmodmap_error (QProcess &proc);
