@@ -43,10 +43,12 @@ PulseManager::PulseManager () : app_dir_ (QApplication::applicationDirPath ()),
                                 pulse_version_minor_ (0),
                                 pulse_version_micro_ (0),
                                 pulse_version_misc_ (""),
+                                pulse_version_valid_ (false),
                                 record_ (true),
                                 playback_ (true),
                                 debug_ (false),
-                                system_pulse_ (false) {
+                                system_pulse_ (false),
+                                shutdown_state_ (false) {
   pulse_dir_ = QDir (QDir::homePath ());
   pulse_dir_.mkpath (pulse_dir_.absolutePath () + pulse_X2Go_ + "/tmp");
   pulse_dir_.cd (pulse_X2Go_.mid (1));
@@ -70,7 +72,7 @@ PulseManager::PulseManager () : app_dir_ (QApplication::applicationDirPath ()),
   QString path_val = tmp_env.value ("PATH");
 
   QStringList to_front, to_back;
-  to_front << "/opt/local/bin"; /* MacPorts default prefix, FIXME: might need to make that configurable. */
+  to_front << MACPORTS_PREFIX "/bin"; /* MacPorts prefix. */
   to_back << "/usr/local/bin"; /* Homebrew or random stuff. Probably even both intermingled... */
 
   path_val = add_to_path (path_val, to_back);
@@ -93,7 +95,7 @@ PulseManager::PulseManager () : app_dir_ (QApplication::applicationDirPath ()),
 
   if (server_binary_.isEmpty ()) {
     search_paths = QStringList ();
-    search_paths << "/opt/local/bin"; /* MacPorts default prefix, FIXME: might need to make that configurable. */
+    search_paths << MACPORTS_PREFIX "/bin"; /* MacPorts prefix. */
 
     server_binary_ = QStandardPaths::findExecutable ("pulseaudio", search_paths);
 
@@ -122,24 +124,14 @@ PulseManager::PulseManager () : app_dir_ (QApplication::applicationDirPath ()),
   }
 #endif /* QT_VERSION < 0x050000 */
 
-  if (server_binary_.isEmpty ()) {
-    x2goErrorf (29) << "Unable to find the PulseAudio binary. Neither bundled, nor found in $PATH nor additional directories.";
-    show_RichText_ErrorMsgBox (tr ("Unable to find the PulseAudio binary. Neither bundled, nor found in $PATH nor additional directories."),
-                               tr ("If you downloaded the bundled, pre-compiled version from the official home page, "
-                                   "please report a bug on:\n"
-                                   "<center><a href=\"https://wiki.x2go.org/doku.php/wiki:bugs\">"
-                                     "https://wiki.x2go.org/doku.php/wiki:bugs"
-                                   "</a></center>\n"),
-                               true);
-    abort ();
+  if (!(server_binary_.isEmpty ())) {
+    QFileInfo tmp_file_info = QFileInfo (server_binary_);
+    server_working_dir_ = tmp_file_info.canonicalPath ();
+
+    x2goDebug << "Found PA binary as " << server_binary_;
+    x2goDebug << "Corresponding working dir: " << server_working_dir_;
   }
 
-  QFileInfo tmp_file_info = QFileInfo (server_binary_);
-  server_working_dir_ = tmp_file_info.canonicalPath ();
-
-  x2goDebug << "Found PA binary as " << server_binary_;
-  x2goDebug << "Corresponding working dir: " << server_working_dir_;
-
 #elif defined (Q_OS_WIN)
   server_working_dir_ = QString (app_dir_ + "/pulse/");
   server_binary_ = QString (app_dir_ + "/pulse/pulseaudio.exe");
@@ -155,7 +147,7 @@ PulseManager::PulseManager () : app_dir_ (QApplication::applicationDirPath ()),
   for (buf = ptr = NULL; ptr == NULL; path_len += 20) {
     if (NULL == (buf = static_cast<char *> (realloc (buf, path_len)))) {
       x2goErrorf (16) << "Could not allocate buffer for getting current working directory!";
-      show_RichText_ErrorMsgBox (tr ("Could not allocate buffer for getting current working directory!"),
+      emit sig_pulse_user_warning (true, tr ("Could not allocate buffer for getting current working directory!"),
                                  QString (),
                                  true);
       abort ();
@@ -167,7 +159,7 @@ PulseManager::PulseManager () : app_dir_ (QApplication::applicationDirPath ()),
     if ((NULL == ptr) && (ERANGE != errno)) {
       int saved_errno = errno;
       x2goErrorf (17) << "getcwd() failed: " << QString (strerror (saved_errno));
-      show_RichText_ErrorMsgBox (tr ("getcwd() failed!"),
+      emit sig_pulse_user_warning (true, tr ("getcwd() failed!"),
                                  QString (strerror (saved_errno)),
                                  true);
       abort ();
@@ -181,7 +173,9 @@ PulseManager::PulseManager () : app_dir_ (QApplication::applicationDirPath ()),
   buf = ptr = NULL;
 #endif // defined (Q_OS_DARWIN)
 
-  fetch_pulseaudio_version ();
+  if (!(server_binary_.isEmpty ())) {
+    fetch_pulseaudio_version ();
+  }
 }
 
 PulseManager::~PulseManager () {
@@ -222,47 +216,46 @@ void PulseManager::start_generic () {
   pulse_server_->setProcessEnvironment (env_);
   pulse_server_->setWorkingDirectory (server_working_dir_);
 
-  pulse_server_->start (server_binary_, server_args_);
+  if (!(server_binary_.isEmpty ())) {
+    pulse_server_->start (server_binary_, server_args_);
 
-  /*
-   * We may wait here, because PulseManager runs in a separate thread.
-   * Otherwise, we'd better use signals and slots to not block the main thread.
-   */
-  if (pulse_server_->waitForStarted (-1)) {
-    x2goDebug << "pulse started with arguments" << server_args_ << "; waiting for finish...";
-    state_ = QProcess::Running;
+    /*
+     * We may wait here, because PulseManager runs in a separate thread.
+     * Otherwise, we'd better use signals and slots to not block the main thread.
+     */
+    if (pulse_server_->waitForStarted (-1)) {
+      x2goDebug << "pulse started with arguments " << server_args_ << "- waiting for it to finish...";
+      state_ = QProcess::Running;
 
-    connect (pulse_server_, SIGNAL (finished (int)),
-             this,          SLOT (slot_on_pulse_finished (int)));
+      connect (pulse_server_, SIGNAL (finished (int)),
+               this,          SLOT (slot_on_pulse_finished (int)));
 
-    env_.insert ("PULSE_SERVER", "127.0.0.1:" + QString::number (pulse_port_));
+      env_.insert ("PULSE_SERVER", "127.0.0.1:" + QString::number (pulse_port_));
 
 
-    QString clean_pulse_dir = pulse_dir_.absolutePath ();
+      QString clean_pulse_dir = pulse_dir_.absolutePath ();
 
 #ifdef Q_OS_WIN
-    clean_pulse_dir = wapiShortFileName (clean_pulse_dir);
+      clean_pulse_dir = wapiShortFileName (clean_pulse_dir);
 #endif /* defined (Q_OS_WIN) */
 
-    QString tmp_auth_cookie = QDir::toNativeSeparators (clean_pulse_dir + "/.pulse-cookie");
+      QString tmp_auth_cookie = QDir::toNativeSeparators (clean_pulse_dir + "/.pulse-cookie");
 
-    env_.insert ("PULSE_COOKIE", tmp_auth_cookie);
+      env_.insert ("PULSE_COOKIE", tmp_auth_cookie);
 
-    if (debug_) {
-      // Give PA a little time to come up.
-      QTimer::singleShot (3000, this, SLOT (slot_play_startup_sound ()));
+      if (debug_) {
+        // Give PA a little time to come up.
+        QTimer::singleShot (3000, this, SLOT (slot_play_startup_sound ()));
+      }
+    }
+    else {
+      x2goErrorf (27) << "PulseAudio failed to start! Sound support will not be available.";
+      show_startup_warning ();
     }
   }
   else {
-    x2goErrorf (27) << "PulseAudio failed to start! Sound support will not be available.";
-    show_RichText_WarningMsgBox (tr ("PulseAudio failed to start!"),
-                                 tr ("Sound support will not be available.\n\n"
-                                     "If you downloaded the bundled, pre-compiled version from the official home page, "
-                                     "please report a bug on:\n"
-                                     "<center><a href=\"https://wiki.x2go.org/doku.php/wiki:bugs\">"
-                                       "https://wiki.x2go.org/doku.php/wiki:bugs"
-                                     "</a></center>\n"),
-                                 true);
+    x2goErrorf (31) << "PulseAudio not detected on system! Sound support will not be available.";
+    show_startup_warning ();
   }
 }
 
@@ -272,6 +265,10 @@ void PulseManager::start_osx () {
                << "-F" << QDir::toNativeSeparators (pulse_dir_.absolutePath () + "/config.pa");
 
   if (!system_pulse_) {
+    if (!(pulse_version_valid_)) {
+      x2goDebug << "PulseAudio version number was not fetched successfully, data is invalid. Continuing anyway.";
+    }
+
     server_args_ << "-p"
                  << QDir::toNativeSeparators (QDir (app_dir_
                                                     + "/../Frameworks/pulse-"
@@ -303,6 +300,11 @@ void PulseManager::start_win () {
  */
 #ifdef Q_OS_WIN
   server_args_ = QStringList ();
+
+  if (!(pulse_version_valid_)) {
+    x2goDebug << "PulseAudio version number was not fetched successfully, data is invalid. Continuing anyway.";
+  }
+
   server_args_ << "--exit-idle-time=-1" << "-n"
                << "-F" << QDir::toNativeSeparators (QDir (pulse_dir_.absolutePath ()
                                                           + "/config.pa").absolutePath ())
@@ -355,6 +357,8 @@ void PulseManager::fetch_pulseaudio_version () {
   tmp_server.setWorkingDirectory (server_working_dir_);
   tmp_server.start (server_binary_, args);
 
+  bool stop_processing = false;
+
   /* Wait until the process exited again. */
   if (tmp_server.waitForFinished ()) {
     /* Read stdout and split it up on newlines. */
@@ -363,7 +367,7 @@ void PulseManager::fetch_pulseaudio_version () {
     QStringList stdout_list (stdout_data.split ("\n"));
 
     bool found = false;
-    for (QStringList::const_iterator cit = stdout_list.begin (); cit != stdout_list.end (); ++cit) {
+    for (QStringList::const_iterator cit = stdout_list.begin (); (cit != stdout_list.end ()) && (!stop_processing); ++cit) {
       /* Remove trailing whitespace, mostly carriage returns on Windows. */
       QString tmp_str (*cit);
       tmp_str = tmp_str.trimmed ();
@@ -380,7 +384,7 @@ void PulseManager::fetch_pulseaudio_version () {
         bool numbers_skip[3] = { false, false, false };
         QString tmp_remaining_str = QString ("");
         QString numbers[3] = { };
-        for (QString::const_iterator cit = tmp_str.begin (); cit != tmp_str.end (); ++cit) {
+        for (QString::const_iterator cit = tmp_str.begin (); (cit != tmp_str.end ()) && (!stop_processing); ++cit) {
           if (!(numbers_finished[0])) {
             if (((*cit) >= '0') && ((*cit) <= '9')) {
               numbers[0].append (*cit);
@@ -404,11 +408,11 @@ void PulseManager::fetch_pulseaudio_version () {
             }
             else {
               x2goErrorf (21) << "Unexpected character found when parsing version string for major version number: '" << QString (*cit) << "'.";
-              show_RichText_ErrorMsgBox (tr ("Error fetching PulseAudio version number!"),
+              emit sig_pulse_user_warning (true, tr ("Error fetching PulseAudio version number!"),
                                          tr ("Unexpected character found when parsing version string for major version number")
                                          + ": '" + QString (*cit) + "'.",
                                          true);
-              abort ();
+              stop_processing = true;
             }
           }
           else if ((!(numbers_finished[1])) && (!(numbers_skip[1]))) {
@@ -429,11 +433,11 @@ void PulseManager::fetch_pulseaudio_version () {
             }
             else {
               x2goErrorf (23) << "Unexpected character found when parsing version string for minor version number: '" << QString (*cit) << "'.";
-              show_RichText_ErrorMsgBox (tr ("Error fetching PulseAudio version number!"),
+              emit sig_pulse_user_warning (true, tr ("Error fetching PulseAudio version number!"),
                                          tr ("Unexpected character found when parsing version string for minor version number")
                                          + ": '" + QString (*cit) + "'.",
                                          true);
-              abort ();
+              stop_processing = true;
             }
           }
           else if ((!(numbers_finished[2])) && (!(numbers_skip[2]))) {
@@ -447,11 +451,11 @@ void PulseManager::fetch_pulseaudio_version () {
             }
             else {
               x2goErrorf (25) << "Unexpected character found when parsing version string for micro version number: '" << QString (*cit) << "'.";
-              show_RichText_ErrorMsgBox (tr ("Error fetching PulseAudio version number!"),
+              emit sig_pulse_user_warning (true, tr ("Error fetching PulseAudio version number!"),
                                          tr ("Unexpected character found when parsing version string for micro version number")
                                          + ": '" + QString (*cit) + "'.",
                                          true);
-              abort ();
+              stop_processing = true;
             }
           }
           else {
@@ -462,11 +466,11 @@ void PulseManager::fetch_pulseaudio_version () {
 
         if (numbers_skip[0]) {
           x2goErrorf (30) << "Supposed to skip major version number. Something is wrong.";
-          show_RichText_ErrorMsgBox (tr ("Error fetching PulseAudio version number!"),
+          emit sig_pulse_user_warning (true, tr ("Error fetching PulseAudio version number!"),
                                      tr ("Supposed to skip major version number. "
                                          "Something is wrong."),
                                      true);
-          abort ();
+          stop_processing = true;
         }
 
         /*     not skipping   and  ((met period or dash) or (have something to convert and met EOL)) */
@@ -476,10 +480,10 @@ void PulseManager::fetch_pulseaudio_version () {
 
           if (!convert_success) {
             x2goErrorf (20) << "Unable to convert major version number string to integer.";
-            show_RichText_ErrorMsgBox (tr ("Error fetching PulseAudio version number!"),
+            emit sig_pulse_user_warning (true, tr ("Error fetching PulseAudio version number!"),
                                        tr ("Unable to convert major version number string to integer."),
                                        true);
-            abort ();
+            stop_processing = true;
           }
           else {
             /* First number is enough to satisfy the "found" criterion. */
@@ -494,10 +498,10 @@ void PulseManager::fetch_pulseaudio_version () {
 
           if (!convert_success) {
             x2goErrorf (22) << "Unable to convert minor version number string to integer.";
-            show_RichText_ErrorMsgBox (tr ("Error fetching PulseAudio version number!"),
+            emit sig_pulse_user_warning (true, tr ("Error fetching PulseAudio version number!"),
                                        tr ("Unable to convert minor version number string to integer."),
                                        true);
-            abort ();
+            stop_processing = true;
           }
         }
 
@@ -508,16 +512,20 @@ void PulseManager::fetch_pulseaudio_version () {
 
           if (!convert_success) {
             x2goErrorf (24) << "Unable to convert micro version number string to integer.";
-            show_RichText_ErrorMsgBox (tr ("Error fetching PulseAudio version number!"),
+            emit sig_pulse_user_warning (true, tr ("Error fetching PulseAudio version number!"),
                                        tr ("Unable to convert micro version number string to integer."),
                                        true);
-            abort ();
+            stop_processing = true;
           }
         }
 
         /* Misc version part will be set to the trailing string. */
         if (found) {
           pulse_version_misc_ = tmp_remaining_str;
+
+          pulse_version_valid_ = true;
+
+          stop_processing = true;
           break;
         }
       }
@@ -528,19 +536,17 @@ void PulseManager::fetch_pulseaudio_version () {
     }
 
     if (!found) {
-      x2goErrorf (19) << "Unable to fetch PulseAudio version - unexpected format. Exiting.";
-      show_RichText_ErrorMsgBox (tr ("Error fetching PulseAudio version number!"),
+      x2goErrorf (19) << "Unable to fetch PulseAudio version - unexpected format.";
+      emit sig_pulse_user_warning (true, tr ("Error fetching PulseAudio version number!"),
                                  tr ("Unexpected format encountered."),
                                  true);
-      abort ();
     }
   }
   else {
-    x2goErrorf (18) << "Unable to start PulseAudio to fetch its version number. Exiting.";
-    show_RichText_ErrorMsgBox (tr ("Error fetching PulseAudio version number!"),
+    x2goErrorf (18) << "Unable to start PulseAudio to fetch its version number.";
+    emit sig_pulse_user_warning (true, tr ("Error fetching PulseAudio version number!"),
                                tr ("Unable to start PulseAudio binary."),
                                true);
-    abort ();
   }
 }
 
@@ -607,7 +613,7 @@ bool PulseManager::generate_server_config () {
      * Tanu Kaskinen recommended we specify the absolute path instead.
      * The absolute path works with at least 5.0 and 6.0.
      */
-    if (pulse_version_major_ > 2) {
+    if ((!(pulse_version_valid_)) || (pulse_version_major_ > 2)) {
       QString clean_pulse_dir = pulse_dir_.absolutePath ();
 
 #ifdef Q_OS_WIN
@@ -762,22 +768,19 @@ void PulseManager::slot_play_startup_sound () {
     }
     else {
       x2goErrorf (26) << "Unable to play startup sound! Something may be wrong.";
-      show_RichText_WarningMsgBox (tr ("Unable to play startup sound."),
-                                   tr ("If you downloaded the bundled, pre-compiled version from the official home page "
-                                       "or the upstream Linux packages, "
-                                       "please report a bug on:\n"
-                                       "<center><a href=\"https://wiki.x2go.org/doku.php/wiki:bugs\">"
-                                         "https://wiki.x2go.org/doku.php/wiki:bugs"
-                                       "</a></center>\n"),
-                                   true);
+      show_startup_warning (true);
     }
   }
 }
 
 void PulseManager::slot_on_pulse_finished (int exit_code) {
-  if (!exit_code)
+  if (exit_code && !shutdown_state_)
+  {
     x2goDebug << "Warning! Pulseaudio's exit code is non-zero.";
-
+    show_startup_warning(true);
+  }
+  shutdown_state_ = false;
+  x2goDebug << "Pulseaudio finished with code:"<<exit_code;
   QByteArray ba (pulse_server_->readAllStandardOutput ());
   char *data = ba.data ();
   x2goDebug << data;
@@ -813,7 +816,7 @@ void PulseManager::slot_on_pulse_finished (int exit_code) {
   emit (sig_pulse_server_terminated ());
 }
 
-bool PulseManager::is_server_running () {
+bool PulseManager::is_server_running () const {
   if (pulse_server_)
     return (pulse_server_->state () == QProcess::Running);
   else
@@ -823,6 +826,7 @@ bool PulseManager::is_server_running () {
 void PulseManager::shutdown () {
   QEventLoop loop;
 
+  shutdown_state_ = true;
   connect (this,  SIGNAL (sig_pulse_server_terminated ()),
            &loop, SLOT (quit ()));
 
@@ -838,23 +842,23 @@ void PulseManager::shutdown () {
   loop.exec ();
 }
 
-std::uint16_t PulseManager::get_pulse_port () {
+std::uint16_t PulseManager::get_pulse_port () const {
   return (pulse_port_);
 }
 
-std::uint16_t PulseManager::get_esd_port () {
+std::uint16_t PulseManager::get_esd_port () const {
   return (esd_port_);
 }
 
-bool PulseManager::get_record () {
+bool PulseManager::get_record () const {
   return (record_);
 }
 
-bool PulseManager::get_playback () {
+bool PulseManager::get_playback () const {
   return (playback_);
 }
 
-QDir PulseManager::get_pulse_dir () {
+QDir PulseManager::get_pulse_dir () const {
   return (pulse_dir_);
 }
 
@@ -884,10 +888,9 @@ bool PulseManager::set_record (bool record) {
   bool ret = false;
 
   if (!(is_server_running ())) {
-    record_ = record;
     ret = true;
   }
-
+  record_ = record;
   return (ret);
 }
 
@@ -917,3 +920,23 @@ void PulseManager::restart () {
 QProcess::ProcessState PulseManager::state () {
   return (state_);
 }
+
+void PulseManager::show_startup_warning (bool play_startup_sound){
+  QString main_text, informative_text;
+
+  if (!(play_startup_sound)) {
+    main_text = tr ("Unable to play startup sound.");
+  }
+  else {
+    main_text = tr ("PulseAudio failed to start!");
+    informative_text = tr ("Sound support will not be available.") + "\n\n";
+  }
+
+  informative_text += tr ("If you downloaded the bundled, pre-compiled version from the official home page "
+                          "or the upstream Linux packages, please report a bug on:\n"
+                          "<center><a href=\"https://wiki.x2go.org/doku.php/wiki:bugs\">"
+                            "https://wiki.x2go.org/doku.php/wiki:bugs"
+                          "</a></center>\n");
+
+  emit sig_pulse_user_warning (false, main_text, informative_text, true);
+}
