@@ -81,7 +81,6 @@ public:
                         QString proxylogin=QString::null, QString proxypassword=QString::null, QString proxyKey=QString::null,
                         bool proxyAutologin=false, bool proxyKrbLogin=false);
     ~SshMasterConnection();
-    static void finalizeLibSsh();
     void addChannelConnection(SshProcess* creator, int sock, QString forwardHost,
                               int forwardPort, QString localHost, int localPort, void* channel=0l);
     void addChannelConnection(SshProcess* creator, QString uuid, QString cmd);
@@ -89,7 +88,7 @@ public:
     void writeKnownHosts(bool);
     void setKeyPhrase(QString);
 
-    int executeCommand(const QString& command, QObject* receiver=0, const char* slotFinished=0);
+    int executeCommand(const QString& command, QObject* receiver=0, const char* slotFinished=0, bool overridePath=true);
     int startTunnel(const QString& forwardHost, uint forwardPort, const QString& localHost,
                     uint localPort, bool reverse=false, QObject* receiver=0, const char* slotTunnelOk=0, const char* slotFinished=0);
     int copyFile(const QString& src, const QString dst, QObject* receiver=0, const char* slotFinished=0);
@@ -122,8 +121,10 @@ private:
     bool userAuthAuto();
     bool userAuthWithKey();
     bool userChallengeAuth();
+    bool checkLogin();
     bool userAuth();
     bool userAuthKrb();
+    bool userAuthKeyboardInteractive(QString prompt);
     void channelLoop();
     void finalize(int arg1);
     void copy();
@@ -147,6 +148,13 @@ private slots:
     void slotSshProxyTunnelOk(int);
     void slotSshProxyTunnelFailed(bool result,  QString output,
                                   int);
+    void slotSshProxyInteractionStart ( SshMasterConnection* connection, QString prompt );
+    void slotSshProxyInteractionUpdate ( SshMasterConnection* connection, QString output );
+    void slotSshProxyInteractionFinish ( SshMasterConnection* connection);
+
+public slots:
+    void interactionTextEnter(QString text);
+    void interactionInterruptSlot();
 
 private:
     ssh_session my_ssh_session;
@@ -158,6 +166,9 @@ private:
     QMutex disconnectFlagMutex;
     QMutex writeHostKeyMutex;
     QMutex reverseTunnelRequestMutex;
+    QMutex interactionInputMutex;
+    QString interactionInputText;
+    bool interactionInterrupt;
     bool writeHostKey;
     bool writeHostKeyReady;
     int nextPid;
@@ -219,7 +230,11 @@ signals:
 
     void needPassPhrase(SshMasterConnection*, bool verificationCode);
     void needChallengeResponse(SshMasterConnection*, QString Challenge);
+    void startInteraction(SshMasterConnection*, QString prompt);
+    void finishInteraction(SshMasterConnection*);
+    void updateInteraction(SshMasterConnection*, QString output);
 };
 
 
 #endif // SSHMASTERCONNECTION_H
+
